----------------------------------------------------------------------------------------
--   Script:		BevelMan
--	 Version:		v0.3
--	 Started:       09/26/2002
--	 Last Modified: 09/29/2002
--   Description:	A Manipulator displayed when working with EditablePoly.
--				 	Allows you to interactively apply Extrude, Outline, Inset and Bevel
--					to the object. Provides controls over Bevel normals mode and let you
--					Start New, Apply current and Cancel the operation. 
--	 Documentation: Visit http://www.scriptspot.com/bobo/darkmoon/bevelman
--	 Code by:		Borislav Petrov [Boboland]		bobo@email.archlab.tuwien.ac.at
----------------------------------------------------------------------------------------
--   IMPORTANT: 	To install correctly, copy this file to your \Plugins directory.
--					Be sure to remove any previous versions of the BevelMan manipulator!
--					To use the new setup options, customize interface by dragging
--					the "BevelManSetup" ActionItem from the Bobo_s Tools Category.
----------------------------------------------------------------------------------------

persistent global bevelMan_Gizmo_Enabled
if bevelMan_Gizmo_Enabled == undefined then bevelMan_Gizmo_Enabled = true

persistent global bevelMan_Gizmo_Size_Type
if bevelMan_Gizmo_Size_Type == undefined then bevelMan_Gizmo_Size_Type = 2

persistent global bevelMan_Gizmo_Size_Rel_Face
if bevelMan_Gizmo_Size_Rel_Face == undefined then bevelMan_Gizmo_Size_Rel_Face = 0.25

persistent global bevelMan_Gizmo_Size_Rel_Outline
if bevelMan_Gizmo_Size_Rel_Outline == undefined then bevelMan_Gizmo_Size_Rel_Outline = 0.25

persistent global bevelMan_Gizmo_Size_Rel_Object
if bevelMan_Gizmo_Size_Rel_Object == undefined then bevelMan_Gizmo_Size_Rel_Object = 0.01

persistent global bevelMan_Gizmo_Size_Abs
if bevelMan_Gizmo_Size_Abs == undefined then bevelMan_Gizmo_Size_Abs = 10.0

persistent global bevelMan_Gizmo_Offset
if bevelMan_Gizmo_Offset == undefined then bevelMan_Gizmo_Offset = 0.0

persistent global bevelMan_Menu_Start
if bevelMan_Menu_Start == undefined then bevelMan_Menu_Start = 15.0

persistent global bevelMan_Menu_Step
if bevelMan_Menu_Step == undefined then bevelMan_Menu_Step = 30.0

persistent global BevelMan_unselectedColor 
if BevelMan_unselectedColor == undefined then BevelMan_unselectedColor = color 0 1 0

persistent global BevelMan_selectedColor 
if BevelMan_selectedColor == undefined then BevelMan_selectedColor = color 1 1 0 

persistent global BevelMan_textColor 
if BevelMan_textColor  == undefined then BevelMan_textColor = color 0 1 1

persistent global BevelMan_textSelectedColor 
if BevelMan_textSelectedColor  == undefined then BevelMan_textSelectedColor = color 1 1 1

persistent global BevelMan_menu_visible
if BevelMan_menu_visible == undefined then BevelMan_menu_visible = true



global BevelMan_last_mode = 0

plugin simpleManipulator EPolyBevelManipulator
    name:"EPolyBevelManipulator"
    invisible:true
(

	local size = 3
	local face_center = [0,0,0]
	local face_normal = [0,0,0]
	local face_array = #()
	local sphere_pos 
	local box_pos
	local base_length
	
    on canManipulate target return (classof target == Editable_Poly and bevelMan_Gizmo_Enabled)  --  and SubObjectLevel > 3
	
	fn updateFaceInfo =
	(
		max_distance = base_length = 0
		in coordsys node 
		(
			face_center = polyOp.getFaceCenter target face_array[1]
			face_normal = polyOp.getFaceNormal target face_array[1] 
		)	
		verts_array = (polyOp.getVertsUsingFace target face_array[1]) as array
		for i in verts_array do
		(
			first_vertex = (polyOp.getVert target verts_array[1])
			base_length = distance first_vertex face_center
			if base_length > max_distance then max_distance = base_length 
		)	
		max_distance
	)
	
    on updateGizmos do
    (
        this.clearGizmos()
		face_center = [0,0,0]
		face_normal = [0,0,0]
		first_vertex = [0,0,0]
		verts_array = #()
		face_array = (polyOp.getFaceSelection target) as array
		if face_array.count > 0 then
		(
			base_length = updateFaceInfo()
			case  bevelMan_Gizmo_Size_Type of
			(
				1:	size = base_length * bevelMan_Gizmo_Size_Rel_Face
				2:	size = (base_length+target.bevelOutline) * bevelMan_Gizmo_Size_Rel_Outline
				3:  size = (distance node.min node.max) * bevelMan_Gizmo_Size_Rel_Object
				4:  size = bevelMan_Gizmo_Size_Abs
			)	
			sphere_pos = face_center+face_normal*(target.BevelHeight+bevelMan_Gizmo_Offset )
		
		
			--DRAW LINE
			giz0 = manip.makeGizmoShape()
	        giz0.addPoint sphere_pos
		    giz0.addPoint face_center	
	        this.addGizmoShape giz0 gizmoDontHitTest BevelMan_unselectedColor BevelMan_selectedColor 
	
			my_tm = (matrixFromNormal face_normal)
			translate my_tm sphere_pos
	
			--DRAW OUTLINE HANDLE
			giz2 = manip.MakeCircle  [0,0,0] (base_length+target.bevelOutline) 16
			giz2.transform my_tm
			this.addGizmoShape giz2 0 BevelMan_unselectedColor BevelMan_selectedColor 

			--DRAW HEIGHT HANDLE
			giz1 = manip.MakeSphere sphere_pos size 12
	        this.addGizmoMesh  giz1 0 BevelMan_unselectedColor BevelMan_selectedColor  

			button_offset = base_length+target.bevelOutline
			
			if BevelMan_menu_visible then
			(

				case BevelMan_last_mode of
				(
					0: (
							button_color1 = BevelMan_textSelectedColor 
							button_color2 = button_color3 = BevelMan_textColor 
						)	
					1: (
							button_color2 = BevelMan_textSelectedColor 
							button_color1 = button_color3 = BevelMan_textColor 
						)	
					2: (
							button_color3 = BevelMan_textSelectedColor 
							button_color1 = button_color2 = BevelMan_textColor 
						)	
					default: (
							button_color1 = button_color2 = button_color3 = BevelMan_textColor 				
						)	
				)--end case			
				
	
				local reset_pos = (sphere_pos + [(button_offset+size*2)*cos (bevelMan_Menu_Start),(button_offset+size*2)*sin (bevelMan_Menu_Start),0]*(matrixFromNormal face_normal))
				local ok_pos = (sphere_pos + [(button_offset+size*2)*cos (bevelMan_Menu_Start+bevelMan_Menu_Step),(button_offset+size*2)*sin (bevelMan_Menu_Start+bevelMan_Menu_Step),0]*(matrixFromNormal face_normal) )
				local cancel_pos = (sphere_pos + [(button_offset+size*2)*cos (bevelMan_Menu_Start+bevelMan_Menu_Step*2),(button_offset+size*2)*sin (bevelMan_Menu_Start+bevelMan_Menu_Step*2),0]*(matrixFromNormal face_normal))
				
				this.addGizmoMarker #hollowBox reset_pos 0 BevelMan_unselectedColor BevelMan_selectedColor 
				this.addGizmoMarker #circle ok_pos 0 BevelMan_unselectedColor BevelMan_selectedColor 
				this.addGizmoMarker #xMarker cancel_pos 0 BevelMan_unselectedColor BevelMan_selectedColor 
			)

			case target.bevelType of
			(
				0: (
						b_type_color1 = BevelMan_textSelectedColor 
						b_type_color2 = b_type_color3 = BevelMan_textColor 
					)	
				1: (
						b_type_color2 = BevelMan_textSelectedColor 
						b_type_color1 = b_type_color3 = BevelMan_textColor 
					)	
				2: (
						b_type_color3 = BevelMan_textSelectedColor 
						b_type_color1 = b_type_color2 = BevelMan_textColor 
					)	
				default: (
						b_type_color3 = b_type_color1 = b_type_color2 = BevelMan_textColor 
					)	

			)--end case	
			
			if face_array.count > 1 and BevelMan_menu_visible then 
			(
				local b_type_pos1 = (sphere_pos - [(button_offset+size*2)*cos (bevelMan_Menu_Start),(button_offset+size*2)*sin (bevelMan_Menu_Start),0]*(matrixFromNormal face_normal) )
				local b_type_pos2 = (sphere_pos -  [(button_offset+size*2)*cos (bevelMan_Menu_Start+bevelMan_Menu_Step),(button_offset+size*2)*sin (bevelMan_Menu_Start+bevelMan_Menu_Step),0]*(matrixFromNormal face_normal))
				local b_type_pos3 = (sphere_pos - [(button_offset+size*2)*cos (bevelMan_Menu_Start+bevelMan_Menu_Step*2),(button_offset+size*2)*sin (bevelMan_Menu_Start+bevelMan_Menu_Step*2),0]*(matrixFromNormal face_normal))
	
				--BEVEL TYPES
				this.addGizmoMarker #diamond  b_type_pos1 0 b_type_color1 BevelMan_selectedColor 
				this.addGizmoMarker #diamond b_type_pos2 0 b_type_color2 BevelMan_selectedColor 
				this.addGizmoMarker #diamond b_type_pos3 0 b_type_color3 BevelMan_selectedColor 
	
				this.addGizmoText "GROUP" b_type_pos1 0 b_type_color1 b_type_color1
				this.addGizmoText "LOCAL" b_type_pos2 0 b_type_color2 b_type_color2
				this.addGizmoText "BY POLY" b_type_pos3 0 b_type_color3 b_type_color3
			)
			
			if BevelMan_menu_visible then
			(
				--DRAW BUTTON TEXT
				this.addGizmoText "NEW" reset_pos 0 button_color1 button_color1
				this.addGizmoText "APPLY" ok_pos 0 button_color2 button_color2
				this.addGizmoText "CANCEL" cancel_pos 0 button_color3 button_color3
			)		
			
		)
	
        return node.name + " Height:" + target.BevelHeight as string +" | Outline:"+ target.bevelOutline as string
    )

	on mouseDown m which do
	(
		case which of
		(
		3: (
			--New
			with redraw off
			(			
				target.previewAccept()
				target.faceExtrudeHeight = target.BevelHeight = 0.0
				target.bevelOutline = target.OutlineAmount = target.InsetAmount = 0
			)	
			target.previewBegin #Bevel
			BevelMan_last_mode = 0
			)
		4: (
			-- Apply
			target.previewAccept()
			target.previewBegin #Bevel
			BevelMan_last_mode = 1
			)
		5:  (
			-- Cancel
			target.previewCancel()
			target.faceExtrudeHeight = target.BevelHeight = 0.0
			target.bevelOutline = target.OutlineAmount = target.InsetAmount = 0
			target.previewBegin #Bevel
			BevelMan_last_mode = 2
			)
		6: 	target.bevelType = 0
		7: 	target.bevelType = 1
		8: 	target.bevelType = 2
		)--end case
	)--end on mouseDown

    on mouseMove m which do
    (
		case which of
		(
			1: (
			        local plane1 = manip.makePlaneFromNormal x_axis sphere_pos
			        local plane2 = manip.makePlaneFromNormal y_axis sphere_pos
			        local projectedPoint = [0,0,0]
			        local viewRay = this.getLocalViewRay m
			        local pl = plane1.mostOrthogonal viewRay plane2
			        local res = pl.intersect viewRay &projectedPoint
					if (res) then target.bevelOutline = target.OutlineAmount = target.InsetAmount = (distance sphere_pos projectedPoint) - base_length 
				)--end 1
			2: (
			        local plane1 = manip.makePlaneFromNormal x_axis sphere_pos
			        local plane2 = manip.makePlaneFromNormal y_axis sphere_pos
			        local projectedPoint = [0,0,0]
			        local viewRay = this.getLocalViewRay m
			        local pl = plane1.mostOrthogonal viewRay plane2
			        local res = pl.intersect viewRay &projectedPoint
					if (res) then 
					(
						val = (distance face_center (projectedPoint) )
						value_sign = dot face_normal (projectedPoint-face_center)
						if value_sign < 0 then val = -val
						target.faceExtrudeHeight = target.BevelHeight = val-bevelMan_Gizmo_Offset
					)	
				)--end 2 
		)--end case
    )
)

global BevelMan_StoreSettingsToDisk 
fn BevelMan_StoreSettingsToDisk =
(
	out_file = createFile ((GetDir #plugcfg)+"/BevelMan.ini")
	format "%\n" bevelMan_Gizmo_Enabled to:out_file
	format "%\n" bevelMan_Gizmo_Size_Type to:out_file
	format "%\n" bevelMan_Gizmo_Size_Rel_Face to:out_file
	format "%\n" bevelMan_Gizmo_Size_Rel_Outline to:out_file
	format "%\n" bevelMan_Gizmo_Size_Rel_Object to:out_file
	format "%\n" bevelMan_Gizmo_Size_Abs to:out_file
	format "%\n" bevelMan_Gizmo_Offset to:out_file
	format "%\n" bevelMan_Menu_Start to:out_file
	format "%\n" bevelMan_Menu_Step to:out_file
	format "%\n" BevelMan_unselectedColor to:out_file
	format "%\n" BevelMan_selectedColor to:out_file
	format "%\n" BevelMan_textColor to:out_file
	format "%\n" BevelMan_textSelectedColor to:out_file
	format "%\n" BevelMan_menu_visible to:out_file
	close out_file
)

global BevelMan_ReStoreSettingsFromDisk 
fn BevelMan_ReStoreSettingsFromDisk =
(
	in_file = OpenFile ((GetDir #plugcfg)+"/BevelMan.ini")
	if in_file != undefined then
	(
		try
		(
			bevelMan_Gizmo_Enabled = execute (readLine in_file)
			bevelMan_Gizmo_Size_Type = execute (readLine in_file)
			bevelMan_Gizmo_Size_Rel_Face = execute (readLine in_file)
			bevelMan_Gizmo_Size_Rel_Outline = execute (readLine in_file)
			bevelMan_Gizmo_Size_Rel_Object = execute (readLine in_file)
			bevelMan_Gizmo_Size_Abs = execute (readLine in_file)
			bevelMan_Gizmo_Offset = execute (readLine in_file)
			bevelMan_Menu_Start = execute (readLine in_file)
			bevelMan_Menu_Step = execute (readLine in_file)
			BevelMan_unselectedColor = execute (readLine in_file)
			BevelMan_selectedColor = execute (readLine in_file)
			BevelMan_textColor = execute (readLine in_file)
			BevelMan_textSelectedColor = execute (readLine in_file)
			BevelMan_menu_visible = execute (readLine in_file)
			close out_file
		)
		catch()	
	)
)

macroScript BevelManSetup category:"BevelMan"
buttontext:"BM Setup" 
tooltip:"Setup - Customize BevelMan Gizmo"
(
rollout BevelMan_Setup "BevelMan Setup"
(
	checkbutton enable_gizmo "BevelMan Gizmo Enabled" checked:bevelMan_Gizmo_Enabled  width:180 highlightcolor:(color 200 255 200)
	group "Gizmo Position"
	(
		spinner offset_gizmo_value "Gizmo Offset In Units:" range:[0.0,1000.0,bevelMan_Gizmo_Offset] scale:1.0 fieldwidth:40
	)	

	group "Gizmo Size"
	(
		dropdownlist extrude_gizmo_size_mode items:#("Relative To Face Diagonal","Relative To Outline Gizmo","Relative To Object Size","Absolute Size In Units") selection:bevelMan_Gizmo_Size_Type 
		spinner extrude_gizmo_size "Gizmo Size:" range:[0.001,100.0,0.25] scale:0.001 fieldwidth:40
	)
	
	group "Menu Items (Round-Round-Baby)"
	(
		spinner menu_start "Menu Start Angle:" range:[-180.0,180.0,bevelMan_Menu_Start] scale:1.0 fieldwidth:40
		spinner menu_step  "Menu Step Angle:" range:[1.0,60.0,bevelMan_Menu_Step] scale:1.0 fieldwidth:40
		checkbox menu_visible "Main Menu Visible" checked:BevelMan_menu_visible
	)
	
	group "Gizmo Colors"
	(
		colorpicker Unselected_Color "Normal Gizmo:" color:(BevelMan_unselectedColor*255) align:#right
		colorpicker Selected_Color "Highlighted Gizmo:" color:(BevelMan_SelectedColor*255) align:#right
		colorpicker Text_Color "Normal Text:" color:(BevelMan_textColor*255) align:#right
		colorpicker textSelected_Color "Highlighted Text:" color:(BevelMan_textSelectedColor*255) align:#right
	)
	hyperlink copyright_and_help " 2002 by Borislav 'Bobo' Petrov" address:"http://www.scriptspot.com/bobo/darkmoon/bevelman"
	
	fn update_gizmo_display =
	(
		if subObjectLevel == 4 and classof $ == Editable_Poly then
		(
			polyOp.setFaceSelection $ (polyOp.getFaceSelection $)
		)
		BevelMan_StoreSettingsToDisk()
	)
	
	fn update_gizmo_size =
	(
		case extrude_gizmo_size_mode.selection of
		(
			1:	extrude_gizmo_size.value = bevelMan_Gizmo_Size_Rel_Face 
			2:	extrude_gizmo_size.value = bevelMan_Gizmo_Size_Rel_Outline 
			3:	extrude_gizmo_size.value = bevelMan_Gizmo_Size_Rel_Object 
			4:	extrude_gizmo_size.value = bevelMan_Gizmo_Size_Abs 
		)	
		update_gizmo_display()	
	)		
	
	on menu_visible changed state do
	(
		BevelMan_menu_visible = state
		manipulatemode = not manipulatemode 
		manipulatemode = not manipulatemode 
		update_gizmo_display()
	)
	
	on Unselected_Color changed col do 
	(
		BevelMan_unselectedColor = col/255.0
		update_gizmo_display()
	)

	on Selected_Color changed col do 
	(
		BevelMan_SelectedColor = col/255.0
		update_gizmo_display()
	)
	
	on Text_Color changed col do 
	(
		BevelMan_TextColor = col/255.0
		update_gizmo_display()
	)	
	on textSelected_Color changed col do 
	(
		BevelMan_textSelectedColor = col/255.0
		update_gizmo_display()
	)	
	
	on enable_gizmo changed state do
	(
		bevelMan_Gizmo_Enabled = state
		manipulatemode = not manipulatemode 
		manipulatemode = not manipulatemode 
	)
	
	on menu_start changed val do
	(
		bevelMan_Menu_Start = val 
		update_gizmo_display()
	)	

	on menu_step changed val do
	(
		bevelMan_Menu_Step = val 
		update_gizmo_display()
	)	
	
	on offset_gizmo_value changed val do
	(
		bevelMan_Gizmo_Offset = val
		update_gizmo_display()	
	)
		
	on extrude_gizmo_size_mode selected itm do 
	(
			bevelMan_Gizmo_Size_Type = itm 
			update_gizmo_size()
	)	
	on extrude_gizmo_size changed val do 
	(
		case extrude_gizmo_size_mode.selection of
		(
			1:	bevelMan_Gizmo_Size_Rel_Face = val
			2:	bevelMan_Gizmo_Size_Rel_Outline = val
			3:	bevelMan_Gizmo_Size_Rel_Object = val
			4:	bevelMan_Gizmo_Size_Abs = val
		)	
		update_Gizmo_Display()
	)	
)
destroyDialog BevelMan_Setup
createDialog BevelMan_Setup 200 380
BevelMan_Setup.update_gizmo_size()
BevelMan_ReStoreSettingsFromDisk()
)

(
BevelMan_ReStoreSettingsFromDisk()
)


macroScript BevelManNew category:"BevelMan" 
buttontext:"BM New" 
tooltip:"Start New Extrusion"
(
	on isEnabled return (selection.count == 1 and (classof selection[1].baseobject) == Editable_Poly and SubObjectLevel > 3)
	on execute do
	(
		with redraw off
		(
			target = selection[1].baseobject
			target.previewAccept()
			target.faceExtrudeHeight = target.BevelHeight = 0.0
			target.bevelOutline = target.OutlineAmount = target.InsetAmount = 0
		)
		target.previewBegin #Bevel
		BevelMan_last_mode = 0
	)	
)

macroScript BevelManApply category:"BevelMan" 
buttontext:"BM Apply" 
tooltip:"Apply and Repeat Current Extrusion"
(
	on isEnabled return (selection.count == 1 and (classof selection[1].baseobject) == Editable_Poly and SubObjectLevel > 3)
	on execute do
	(
		target = selection[1].baseobject
		target.previewAccept()
		target.previewBegin #Bevel
		BevelMan_last_mode = 1
	)	
)

macroScript BevelManCancel category:"BevelMan" 
buttontext:"BM Cancel" 
tooltip:"Cancel Current Extrusion"
(
	on isEnabled return (selection.count == 1 and (classof selection[1].baseobject) == Editable_Poly and SubObjectLevel > 3)
	on execute do
	(
		target = selection[1].baseobject
		target.previewCancel()
		target.faceExtrudeHeight = target.BevelHeight = 0.0
		target.bevelOutline = target.OutlineAmount = target.InsetAmount = 0
		target.previewBegin #Bevel
		BevelMan_last_mode = 2
	)	
)

macroScript BevelManGroup category:"BevelMan" 
buttontext:"BM Group" 
tooltip:"Extrude Using Group Normal"
(
	on isEnabled return (selection.count == 1 and (classof selection[1].baseobject) == Editable_Poly and SubObjectLevel > 3)
	on execute do
	(
		target = selection[1].baseobject
		target.bevelType = 0
	)	
)

macroScript BevelManLocal category:"BevelMan" 
buttontext:"BM Local" 
tooltip:"Extrude Using Local Normals"
(
	on isEnabled return (selection.count == 1 and (classof selection[1].baseobject) == Editable_Poly and SubObjectLevel > 3)
	on execute do
	(
		target = selection[1].baseobject
		target.bevelType = 1
	)	
)

macroScript BevelManByPoly category:"BevelMan" 
buttontext:"BM ByPoly" 
tooltip:"Extrude Using Polygon Normals"
(
	on isEnabled return (selection.count == 1 and (classof selection[1].baseobject) == Editable_Poly and SubObjectLevel > 3)
	on execute do
	(
		target = selection[1].baseobject
		target.bevelType = 2
	)	
)


